// Rob Trew 2015 (c) MIT 
// (for reference - expanded form of Javascript used in Keyboard Maestro macro)
// http://forum.keyboardmaestro.com/t/list-all-sentences-containing-a-search-term-in-km-docs-as-md-html/1413
(function (strPath, strTerm) {
	"use strict";
	var strBaseURL =
		"http://www.keyboardmaestro.com/documentation/6/keyboardmaestro.html";

	// The id and text of the parent anchor for this helpfile node:
	// (Most recent (if any) sibling or preceding ancestral sibling with class='anchor')
	// (or ... 'move up and back till we hit anchor or Content root')

	// oDOMnode --> [strParentalAnchorID, strAnchorLegend]
	function matchContext(m) {
		var a = m,
			p;

		if (a.className === 'anchor') {
			return [a.getAttribute('id'), a.textContent];
		} else if (a.className === 'a') {
			return [a.href, a.text];
		}
		while ((p = a.previousSibling) && (a = p)) {
			if (a.className === 'anchor')
				return [a.getAttribute('id'), a.textContent];
		}
		if ((p = a.parentNode) && (a = p)) {
			if (a.getAttribute('id') !== 'Content')
				return matchContext(a);
			else return ["", ""];
		}
		return [a.getAttribute('id'), a.text];
	}
	// XPathResults collection for supplied Path
	var r = document.evaluate(strPath, document, null, 0, null),
		lst = [],
		rgxTerm = new RegExp('(' + strTerm.trim() + ')(s|es)?', 'ig'),
		n = 0, // paragraph number
		sn = 0, // section number
		oNode, mblstAnchor, strText, strURL, strPrevURL;

	// write out each para containing a match
	// only preceding it with an anchor link when the url has just changed
	while (oNode = r.iterateNext()) {
		// tag term with a class tag for CSS highlighting
		strText = '- ' + oNode.textContent.replace(
			rgxTerm, "<span class='term'>$1$2</span>"
		);

		// what is the nearest preceding anchor for this node ?
		mblstAnchor = matchContext(oNode);

		// write out the para, preceding it with new anchor link if changed
		strURL = mblstAnchor ? mblstAnchor[0] : '';
		if (strURL && (strURL !== strPrevURL)) {
			lst.push(
				'\n[' + mblstAnchor[1] + '](' + strBaseURL +
				'#' + strURL + ' "' + strURL + '")\n\n' + strText
			);
			strPrevURL = strURL;
			sn++;
		} else lst.push(strText);
		n++;
	}

	return [
		'<style>' +
		'.term {color:red} ' +
		'.headterm {text-transform: uppercase; font-weight: bolder}' +
		'.results {font-size: 75%; color:gray} ' +
		'</style>',
		'### ' + document.title + '\n',
		"<span class='headterm'>" + strTerm + "</span>" +
		"\t<span class='results'><br>( " + n.toString() +
		" results under " + sn.toString() + " headings )</span>",
		lst.join('\n')
	].join('\n');
})(document.kmvar.XPath, document.kmvar.Term_in_Help_file)